==> wikbook.core-0.9.3-sources.jar: org/wikbook/wiki/WikiLoader.java <==
/*
 * Copyright (C) 2010 eXo Platform SAS.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.wikbook.wiki;

import org.wikbook.WikbookException;
import org.wikbook.ResourceType;
import org.wikbook.Utils;
import org.wikbook.WikletContext;
import org.xwiki.component.embed.EmbeddableComponentManager;
import org.xwiki.component.manager.ComponentLookupException;
import org.xwiki.rendering.block.Block;
import org.xwiki.rendering.block.HeaderBlock;
import org.xwiki.rendering.block.MacroBlock;
import org.xwiki.rendering.block.SectionBlock;
import org.xwiki.rendering.block.VerbatimBlock;
import org.xwiki.rendering.block.WordBlock;
import org.xwiki.rendering.block.XDOM;
import org.xwiki.rendering.listener.HeaderLevel;
import org.xwiki.rendering.parser.ParseException;
import org.xwiki.rendering.parser.Parser;
import org.xwiki.rendering.renderer.BlockRenderer;
import org.xwiki.rendering.renderer.printer.DefaultWikiPrinter;
import org.xwiki.rendering.renderer.printer.WikiPrinter;
import org.xwiki.rendering.syntax.Syntax;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * @author <a href="mailto:julien.viet@exoplatform.com">Julien Viet</a>
 * @version $Revision$
 */
public class WikiLoader
{

   /** . */
   private final WikletContext context;

   /** . */
   private final EmbeddableComponentManager ecm;

   public WikiLoader(WikletContext context) throws WikbookException
   {
      EmbeddableComponentManager ecm = new EmbeddableComponentManager();
      ecm.initialize(Thread.currentThread().getContextClassLoader());

      //
      this.context = context;
      this.ecm = ecm;
   }

   public XDOM load(String id, String syntaxId) throws WikbookException
   {
      return load(id, syntaxId, 0);
   }

   public XDOM load(Reader reader, String syntaxId) throws WikbookException
   {
      return load(reader, syntaxId, 0);
   }

   private XDOM load(String id, String syntaxId, int baseLevel) throws WikbookException
   {
      try
      {
         Reader reader = _load(id);
         return load(reader, syntaxId, baseLevel);
      }
      catch (IOException e)
      {
         throw new WikbookException(e);
      }
   }

   private XDOM load(Reader reader, String syntaxId, int baseLevel) throws WikbookException
   {
      try
      {
         if (syntaxId == null)
         {
            syntaxId = Syntax.XWIKI_2_0.toIdString();
         }

         //
         XDOM xdom;
         if ("verbatim".equals(syntaxId))
         {
            xdom = new XDOM(new ArrayList<Block>());
            xdom.addChild(new VerbatimBlock(Utils.read(reader), false));
         }
         else
         {
            // Get parser
            Parser parser = ecm.lookup(Parser.class, syntaxId);


            // Parse
            xdom = parser.parse(reader);

            // Replace all headers
            if (baseLevel > 0)
            {
               for (HeaderBlock header : xdom.getChildrenByType(HeaderBlock.class, true)) {
                  int l = baseLevel + header.getLevel().getAsInt();
                  HeaderBlock nheader = new HeaderBlock(
                     header.getChildren(),
                     HeaderLevel.parseInt(l),
                     header.getParameters(),
                     header.getId()
                  );
                  header.getParent().replaceChild(nheader, header);
               }
            }

            //
            List<Substitution> substitutions = visit(xdom);

            //
            if (substitutions.size() > 0)
            {
               for (Substitution substitution : substitutions)
               {
                  substitution.src.getParent().replaceChild(substitution.dst, substitution.src);
               }
               WikiPrinter printer = new DefaultWikiPrinter();
               BlockRenderer renderer = ecm.lookup(BlockRenderer.class, Syntax.XWIKI_2_0.toIdString());
               renderer.render(xdom, printer);
               xdom = parser.parse(new StringReader(printer.toString()));
            }
         }

         //
         return xdom;
      }
      catch (IOException e)
      {
         throw new WikbookException(e);
      }
      catch (ComponentLookupException e)
      {
         throw new WikbookException(e);
      }
      catch (ParseException e)
      {
         throw new WikbookException(e);
      }
   }

   /**
    * Load the document with the specified id.
    *
    * @param id the document id
    * @return a reader for the document
    * @throws IOException any io exception
    */
   private Reader _load(String id) throws IOException
   {
      URL main = context.resolveResource(ResourceType.WIKI_SOURCE, id);
      return Utils.read(main);
   }

   /**
    * Visit a block and returns the list of substitutions that must be performed in the visited block.
    * The returned list can be safely modified.
    *
    * @param block the block to visit
    * @return the substitution list
    */
   private List<Substitution> visit(Block block)
   {
      return visit(block, 0);
   }

   private List<Substitution> visit(Block block, int level)
   {
      if (block instanceof MacroBlock)
      {
         MacroBlock macro = (MacroBlock)block;
         String id = macro.getId();
         if ("include".equals(id))
         {
            String includedId = macro.getParameter("document");
            String syntax = macro.getParameter("syntax");
            LinkedList<Substitution> list = new LinkedList<Substitution>();
            list.add(new Substitution(block, load(includedId, syntax, level)));
            return list;
         }
         else if (id.startsWith("property."))
         {
            String name = id.substring("property.".length());
            if (name != null)
            {
               String value = context.getProperty(name);
               if (value != null)
               {
                  LinkedList<Substitution> list = new LinkedList<Substitution>();
                  list.add(new Substitution(block, new WordBlock(value)));
                  return list;
               }
            }
         }
      }
      else if (block instanceof SectionBlock)
      {
         level++;
      }

      //
      List<Substitution> substitutions = new LinkedList<Substitution>();
      for (Block child : block.getChildren())
      {
         List<Substitution> childSubstitutions = visit(child, level);
         if (childSubstitutions.size() > 0)
         {
            substitutions.addAll(childSubstitutions);
         }
      }

      //
      return substitutions;
   }
}

==> wikbook.core-0.9.5-sources.jar: org/wikbook/wiki/WikiLoader.java <==
/*
 * Copyright (C) 2010 eXo Platform SAS.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.wikbook.wiki;

import org.wikbook.WikbookException;
import org.wikbook.ResourceType;
import org.wikbook.Utils;
import org.wikbook.WikletContext;
import org.xwiki.component.embed.EmbeddableComponentManager;
import org.xwiki.component.manager.ComponentLookupException;
import org.xwiki.rendering.block.Block;
import org.xwiki.rendering.block.HeaderBlock;
import org.xwiki.rendering.block.MacroBlock;
import org.xwiki.rendering.block.SectionBlock;
import org.xwiki.rendering.block.VerbatimBlock;
import org.xwiki.rendering.block.WordBlock;
import org.xwiki.rendering.block.XDOM;
import org.xwiki.rendering.listener.HeaderLevel;
import org.xwiki.rendering.parser.ParseException;
import org.xwiki.rendering.parser.Parser;
import org.xwiki.rendering.renderer.BlockRenderer;
import org.xwiki.rendering.renderer.printer.DefaultWikiPrinter;
import org.xwiki.rendering.renderer.printer.WikiPrinter;
import org.xwiki.rendering.syntax.Syntax;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * @author <a href="mailto:julien.viet@exoplatform.com">Julien Viet</a>
 * @version $Revision$
 */
public class WikiLoader
{

   /** . */
   private final WikletContext context;

   /** . */
   private final EmbeddableComponentManager ecm;

   public WikiLoader(WikletContext context) throws WikbookException
   {
      EmbeddableComponentManager ecm = new EmbeddableComponentManager();
      ecm.initialize(Thread.currentThread().getContextClassLoader());

      //
      this.context = context;
      this.ecm = ecm;
   }

   public XDOM load(String id, String syntaxId) throws WikbookException
   {
      return load(id, syntaxId, 0);
   }

   public XDOM load(Reader reader, String syntaxId) throws WikbookException
   {
      return load(reader, syntaxId, 0);
   }

   private XDOM load(String id, String syntaxId, int baseLevel) throws WikbookException
   {
      try
      {
         Reader reader = _load(id);
         return load(reader, syntaxId, baseLevel);
      }
      catch (IOException e)
      {
         throw new WikbookException(e);
      }
   }

   private XDOM load(Reader reader, String syntaxId, int baseLevel) throws WikbookException
   {
      try
      {
         if (syntaxId == null)
         {
            syntaxId = Syntax.XWIKI_2_0.toIdString();
         }

         //
         XDOM xdom;
         if ("verbatim".equals(syntaxId))
         {
            xdom = new XDOM(new ArrayList<Block>());
            xdom.addChild(new VerbatimBlock(Utils.read(reader), false));
         }
         else
         {
            // Get parser
            Parser parser = ecm.lookup(Parser.class, syntaxId);


            // Parse
            xdom = parser.parse(reader);

            // Replace all headers
            if (baseLevel > 0)
            {
               for (HeaderBlock header : xdom.getChildrenByType(HeaderBlock.class, true)) {
                  int l = baseLevel + header.getLevel().getAsInt();
                  HeaderBlock nheader = new HeaderBlock(
                     header.getChildren(),
                     HeaderLevel.parseInt(l),
                     header.getParameters(),
                     header.getId()
                  );
                  header.getParent().replaceChild(nheader, header);
               }
            }

            //
            List<Substitution> substitutions = visit(xdom);

            //
            if (substitutions.size() > 0)
            {
               for (Substitution substitution : substitutions)
               {
                  substitution.src.getParent().replaceChild(substitution.dst, substitution.src);
               }
               WikiPrinter printer = new DefaultWikiPrinter();
               BlockRenderer renderer = ecm.lookup(BlockRenderer.class, Syntax.XWIKI_2_0.toIdString());
               renderer.render(xdom, printer);
               xdom = parser.parse(new StringReader(printer.toString()));
            }
         }

         //
         return xdom;
      }
      catch (IOException e)
      {
         throw new WikbookException(e);
      }
      catch (ComponentLookupException e)
      {
         throw new WikbookException(e);
      }
      catch (ParseException e)
      {
         throw new WikbookException(e);
      }
   }

   /**
    * Load the document with the specified id.
    *
    * @param id the document id
    * @return a reader for the document
    * @throws IOException any io exception
    */
   private Reader _load(String id) throws IOException
   {
      URL main = context.resolveResource(ResourceType.WIKI, id);
      return Utils.read(main);
   }

   /**
    * Visit a block and returns the list of substitutions that must be performed in the visited block.
    * The returned list can be safely modified.
    *
    * @param block the block to visit
    * @return the substitution list
    */
   private List<Substitution> visit(Block block)
   {
      return visit(block, 0);
   }

   private List<Substitution> visit(Block block, int level)
   {
      if (block instanceof MacroBlock)
      {
         MacroBlock macro = (MacroBlock)block;
         String id = macro.getId();
         if ("include".equals(id))
         {
            String includedId = macro.getParameter("document");
            String syntax = macro.getParameter("syntax");
            LinkedList<Substitution> list = new LinkedList<Substitution>();
            list.add(new Substitution(block, load(includedId, syntax, level)));
            return list;
         }
         else if (id.startsWith("property."))
         {
            String name = id.substring("property.".length());
            if (name != null)
            {
               String value = context.getProperty(name);
               if (value != null)
               {
                  LinkedList<Substitution> list = new LinkedList<Substitution>();
                  list.add(new Substitution(block, new WordBlock(value)));
                  return list;
               }
            }
         }
      }
      else if (block instanceof SectionBlock)
      {
         level++;
      }

      //
      List<Substitution> substitutions = new LinkedList<Substitution>();
      for (Block child : block.getChildren())
      {
         List<Substitution> childSubstitutions = visit(child, level);
         if (childSubstitutions.size() > 0)
         {
            substitutions.addAll(childSubstitutions);
         }
      }

      //
      return substitutions;
   }
}
